--[[
Plugin for range check
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("RangeCheck")

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Range")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local Party
local Utils = RW.Utils
local db
local dragStart, dragStop
local Anchor, Window, ConfigWindow, ConfigPosX, ConfigPosY
local InConfigMode
local LockButton, FilterButton
local Update

local LocalFilter = {
	UseNameFilter = false,
	FilteredName = {}
}

local ActiveBars = {}
local AllBars = {}
local barProt = {}

local iconCoords = {
	{0,		0.25,	0, 		0.25},
	{0.25,	0.5,	0,		0.25},
	{0.5, 	0.75,	0,		0.25},
	{0.75,	1,		0,		0.25},
	{0,		0.25,	0.25,	0.5},
	{0.25,	0.5,	0.25,	0.5},
	{0.5,	0.75,	0.25,	0.5},
	{0.75,	1,		0.25,	0.5},
}

local classes = {}
FillLocalizedClassList(classes, true)


local ICON_COORDS = {}
local e = 0.02
for class,coords in pairs(CLASS_ICON_TCOORDS) do
	local l,r,t,b = unpack(coords)
	ICON_COORDS[class] = {l+e, r-e, t+e, b-e}
end

local defaults = {
	profile = {
		UpdateFreq = 0.05,
		PluginEnabled = true,
		Width = 250,
		Locked = false,
		NoAutoHide = false,
		
		Alpha = 1,
		CloseBottom = true,
		
		
		-- Visual Window
		Border = "None",
		BorderWidth = 10,
		BorderColor =  {1, 1, 1, 1},
		Inset = 3,
		UseGradient = true,
		
		-- Visual Bars
		BarTexture = "HalD",
		BarBackgroundColor = {0, 0, 0, 0.5},
		BarAlpha = 1.0,
		DistanceFont = "accid",
		DistanceFontSize = 18,
		DistanceFontFlags = "OUTLINE",
		DistanceFontColor = {1, 0.3, 0, 1},
		NameFont = "accid",
		NameFontSize = 16,
		NameFontFlags = "OUTLINE",
		NameFontColor = {1, 1, 1, 1},
		UseClassColor = true,
		BarColor = {111/255, 146/255, 178/255, 1},
		ShowIcon = true,
		ShowRaidIcon = true,
		IconSide = "LEFT",
		Reverse = false,
		
		-- Names
		NrNames = 5,
		NamesHeight = 35,
		Spacing = 1,
		range = 15,
		maxNames = 5,
		GrowUp = true,
		x = 0,
		y = 0,
		
		-- Filter
		Filter = {
			icons = {},
			classes = {},			
			UseIconFilter = false,
			UseClassFilter = false,
		}
	}
}

for class in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	defaults.profile.Filter.classes[class] = true
end

for i = 1, 8 do
	defaults.profile.Filter.icons[i] = true
end

local backdrop = {
}

--------------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("RangeCheck", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_ShowRangeCheck")
	self:RegisterMessage("RW_HideRangeCheck")
	self:RegisterMessage("RW_RangeFilterIcons")
	self:RegisterMessage("RW_RangeFilterClasses")
	self:RegisterMessage("RW_RangeFilterNames")
	self:RegisterMessage("RW_RangeFilterClear")
	self:RegisterMessage("RW_RangeFilterAddIcon")
	self:RegisterMessage("RW_RangeFilterAddName")
	self:RegisterMessage("RW_RangeFilterRemoveName")
	
	Party = RW:GetPlugin("Party")
	
	if not Anchor then
		self:CreateAnchor()
	end
	
	self:Reset()
end

function plugin:PluginDisable()
	Anchor:Hide()
end

function plugin:Reset()
	db = self.db.profile
	
	Anchor:ClearAllPoints()
	Anchor:SetPoint("CENTER", db.x, db.y)
	Anchor:SetWidth(db.Width)
	
	if Window then
		Window:ClearAllPoints()
		Window:SetPoint("BOTTOM", Anchor, "TOP")
		Window:SetBorder(LSM:Fetch("border", db.Border), db.BorderWidth, db.BorderColor)
		Window:SetInsets(db.Border == "None" and 0 or db.Inset)
		if db.Border == "None" then
			Window:SetContentHeight(db.NrNames * db.NamesHeight + (db.NrNames - 1) * db.Spacing + 1)
		else
			Window:SetContentHeight(db.NrNames * db.NamesHeight + (db.NrNames - 1) * db.Spacing + 2 * db.Inset)
		end
		Window:SetWidth(db.Width)	

		Window:SetAlpha(db.Alpha)
		
		while #AllBars < 40 do
			self:CreateBar()
		end
		
		for i, bar in pairs(AllBars) do			
			local t = i - 1
			bar.frame:ClearAllPoints()
			bar.frame:SetPoint("TOPLEFT", Window.title, "BOTTOMLEFT", 2, -(db.Spacing * t) - (db.NamesHeight * t))
			bar.frame:SetPoint("TOPRIGHT", Window.title, "BOTTOMRIGHT", -2, -(db.Spacing * t) - (db.NamesHeight * t))
			bar:ApplySettings()
		end
	end	
	
	if FilterButton then
		if db.Filter.UseClassFilter or db.Filter.UseIconFilter or LocalFilter.UseNameFilter then
			FilterButton.UpTexture:SetVertexColor(0, 1, 0, 1)
		else
			FilterButton.UpTexture:SetVertexColor(unpack(RW.db.profile.WindowButtonColor))
		end
	end
end

------------------------------------
-- Anchor
function plugin:CreateAnchor()
	Anchor = Utils:CreateAnchor("RangeCheck", UIParent, L.NAME)
	
	Anchor:SetScript("OnDragStart", dragStart)
	Anchor:SetScript("OnDragStop", dragStop)	
end

------------------------------------
-- Window
function plugin:CreateWindow()
	Window = Utils:CreateWindow("RangeCheck")
	self:SetTitle()
	
	Window:SetScript("OnUpdate", Update)
	
	local b = Window:AddTitleButton("CloseUp", "CloseDown")	
	b:AddClickHandler( function() Window:Hide() end)
	Utils:AddTooltip(b, LB.CLOSE)
	
	b = Window:AddTitleButton("ConfigUp", "ConfigDown")
	b:AddClickHandler( function()
		plugin:OpenConfigWindow()
	end)
	Utils:AddTooltip(b, LB.CONFIGURE)
	
	b = Window:AddTitleButton("UpPointerUp", "UpPointerDown")
	b:AddClickHandler(function() plugin:IncreaseRange() end)
	Utils:AddTooltip(b, L.INCREASE_DECREASE_DESC, L.INCREASE_RANGE)
	
	b = Window:AddTitleButton("DownPointerUp", "DownPointerDown")
	b:AddClickHandler(function() plugin:DecreaseRange() end)
	Utils:AddTooltip(b, L.INCREASE_DECREASE_DESC, L.DECREASE_RANGE)
	
	b = Window:AddTitleButton("LockUp", "LockDown")
	if db.Locked then
		b.UpTexture:SetVertexColor(1, 0, 0)
	end
	b:SetScript("OnLeave", function(self)
		if db.Locked then
			self.UpTexture:SetVertexColor(1, 0, 0)
		else
			self.UpTexture:SetVertexColor(unpack(RW.db.profile.WindowButtonColor))
		end
	end)
	b:AddClickHandler(function()
		db.Locked = not db.Locked
		if db.Locked then
			plugin:Msg("Locked")
		else
			plugin:Msg("Unlocked")
		end
		if ConfigWindow then
			plugin:OpenConfigWindow(true)
		end
	end)
	Utils:AddTooltip(b, L.LOCK_DESC, L.LOCK)
	LockButton = b
	
	b = Window:AddTitleButton("FilterUp", "FilterDown")
	b:AddClickHandler(function()
		plugin:ResetFilters()
		if ConfigWindow then
			plugin:OpenConfigWindow(true)
		end
	end)
	b:SetScript("OnLeave", function(self)
		if db.Filter.UseClassFilter or db.Filter.UseIconFilter or LocalFilter.UseNameFilter then
			self.UpTexture:SetVertexColor(0, 1, 0, 1)
		else
			self.UpTexture:SetVertexColor(unpack(RW.db.profile.WindowButtonColor))
		end
	end)
	Utils:AddTooltip(b, L.FILTER_ACTIVE_DESC, L.FILTER_ACTIVE)
	FilterButton = b
	
	self:Reset()
end

function plugin:OpenConfigWindow(toggle)
	local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	if not ConfigWindow then
		ConfigWindow = UIF:Frame(L.NAME)	
		ConfigWindow:SetHeight(800)
		ConfigWindow:SetCallback("OnClose", function()
			ConfigWindow:ReleaseChildren()
			ConfigWindow.frame:Hide()
		end)
		ConfigWindow:SetLayout("Fill")		
		ConfigWindow:Hide()
	end
	
	if toggle then
		if ConfigWindow.frame:IsShown() then
			ConfigWindow:Hide()
			ConfigWindow:Show()
			ConfigWindow:AddChildren(plugin:GetGUI())	
		end
	else
		if ConfigWindow.frame:IsShown() then
			ConfigWindow:Hide()
		else		
			ConfigWindow:Show()
			ConfigWindow:AddChildren(plugin:GetGUI())		
		end
	end
end

------------------------------------
-- Bars
function plugin:CreateBar()
	local frame = CreateFrame("frame", nil, Window)
	
	local bg = frame:CreateTexture(nil, "BACKGROUND")
	
	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetFrameLevel(Window.content:GetFrameLevel() + 1)
	bar:SetMinMaxValues(0, 1)
	
	local raidicon = bar:CreateTexture(nil, "OVERLAY")
	raidicon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp")
	raidicon:Hide()
	
	local icon = bar:CreateTexture(nil, "ARTWORK")
	icon:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
	
	local playertext = bar:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	playertext:SetShadowOffset(1, -1)
	playertext:SetJustifyH("LEFT")
	
	local distancetext = bar:CreateFontString(nil, "ARTOWRK", "GameFontNormal")
	distancetext:SetShadowOffset(1, -1)
	distancetext:SetJustifyH("RIGHT")

	newBar = setmetatable({
		frame = frame,
		bar = bar,
		background = bg,
		icon = icon,
		raidicon = raidicon,
		playertext = playertext,
		distancetext = distancetext,
		id = time() + math.random(),
	}, {__index = barProt})
	
	tinsert(AllBars, newBar)
	
	frame:Hide()
	
	return newBar
end

function barProt:Hide()
	self.frame:Hide()
end

function barProt:Show()
	self.frame:Show()
end

function barProt:SetStatusBarColor(...)
	self.bar:SetStatusBarColor(...)
end

function barProt:SetValue(value)
	self.bar:SetValue(value)
end

function barProt:ApplySettings()
	-- Clear positions
	self.playertext:ClearAllPoints()
	self.distancetext:ClearAllPoints()
	self.bar:ClearAllPoints()
	self.icon:ClearAllPoints()
	self.background:ClearAllPoints()
	
	-- Set textures
	self.background:SetTexture(unpack(db.BarBackgroundColor))
	self.bar:SetStatusBarTexture(LSM:Fetch("statusbar", db.BarTexture))
	
	-- Set Colors
	self.bar:GetStatusBarTexture():SetHorizTile(false)
	if not db.UseClassColor then
		self.bar:SetStatusBarColor(unpack(db.BarColor))
	end
	
	self.playertext:SetTextColor(unpack(db.NameFontColor))
	self.distancetext:SetTextColor(unpack(db.DistanceFontColor))
	
	-- Set Fonts
	self.playertext:SetFont(LSM:Fetch("font", db.NameFont), db.NameFontSize, db.NameFontFlags)
	self.distancetext:SetFont(LSM:Fetch("font", db.DistanceFont), db.DistanceFontSize, db.DistanceFontFlags)
	
	-- Set Points
	self.frame:SetHeight(db.NamesHeight)
	self.background:SetPoint("TOPLEFT")
	self.background:SetPoint("BOTTOMRIGHT")
	self.raidicon:SetPoint("CENTER")
	self.raidicon:SetHeight(db.NamesHeight - 4)
	self.raidicon:SetWidth(db.NamesHeight - 4)
	
	if db.ShowIcon and self.icon:GetTexture() then
		self.icon:Show()
		
		if db.IconSide == "LEFT" then		
			self.icon:SetPoint("TOPLEFT", self.background)			
			
			self.bar:SetPoint("TOPLEFT", self.icon, "TOPRIGHT", 1, 0)
			self.bar:SetPoint("BOTTOMRIGHT", self.background)
		else
			self.icon:SetPoint("TOPRIGHT", self.background)
			
			self.bar:SetPoint("TOPLEFT", self.background)
			self.bar:SetPoint("BOTTOMRIGHT", self.icon, "BOTTOMLEFT", -1, 0)
		end
		
		self.icon:SetPoint("BOTTOM", self.background)
		self.icon:SetWidth(db.NamesHeight)
	else
		self.icon:Hide()
		self.bar:SetPoint("TOPLEFT", self.background)
		self.bar:SetPoint("BOTTOMRIGHT", self.background)
	end
	

	self.distancetext:SetPoint("RIGHT", self.bar, -5, 0)
	
	self.playertext:SetPoint("LEFT", 5, 0)
	self.playertext:SetPoint("RIGHT", self.distancetext, -5, 0)	
end

local function BarSort(a, b)
	if not a.range then return false end
	if not b.range then return true end
	return tonumber(a.range) > tonumber(b.range)
end

local NameList = {}
Update = function(self, elapsed)
	self.t = (self.t or 0) + elapsed
	if self.t > db.UpdateFreq then
		self.t = 0
		local count = 0
		for name, data in Party:PartyIterator() do
			local ShowFlag = count < db.NrNames
			wipe(NameList)
			if (not UnitIsUnit("player", data.id)) and not UnitIsDeadOrGhost(data.id) and count < 40 then
				
				-- Check Raid Icon filter
				local icon = GetRaidTargetIndex(data.id)
				local filterResult = false
				local isFiltered = db.Filter.UseIconFilter or db.Filter.UseClassFilter or LocalFilter.UseNameFilter
				if db.Filter.UseIconFilter then	
					if not icon then 
						filterResult = false
					elseif db.Filter.icons[icon] then 
						filterResult = true 
					else
						filterResult = false
					end
				end
				
				-- Check Class filter
				if db.Filter.UseClassFilter then
					if db.Filter.classes[data.class] then filterResult = true end
				end
				
				-- Check Name filter
				if LocalFilter.UseNameFilter then
					if LocalFilter.FilteredName[name] then filterResult = true end
				end
				
				if not isFiltered or (isFiltered and filterResult) then
					local inRange, range = plugin:CheckRange(data.id)
					
					if inRange then						
						count = count + 1						
						local bar = AllBars[count]
						bar.range = range
						
						if db.UseClassColor then
							local c = RAID_CLASS_COLORS[data.class]
							AllBars[count]:SetStatusBarColor(c.r, c.g, c.b, db.BarAlpha)
						end
						
						local val = range / db.range
						if db.Reverse then val = 1 - val end
						bar:SetValue(range < db.range and val or 1)
						bar.playertext:SetText(name)
						bar.distancetext:SetText(("%.1f"):format(range))
						
						if icon and db.ShowRaidIcon then
							bar.raidicon:SetTexCoord(unpack(iconCoords[icon]))
							bar.raidicon:Show()
						else
							bar.raidicon:Hide()
						end
						
						if db.ShowIcon then
							bar.icon:SetTexCoord(unpack(ICON_COORDS[data.class]))
							bar.icon:Show()
						else
							bar.icon:Hide()
						end
						
						bar.randomset = nil
						bar.randomrange = nil
					end
				end				
			end
		end
		
		-- Config mode
		if InConfigMode and count < db.NrNames then
			while count < db.NrNames do
				count = count + 1
				local bar = AllBars[count]
				local randClass = math.random(8)
				
				if db.UseClassColor then
					local c = RAID_CLASS_COLORS[CLASS_SORT_ORDER[randClass]]
					if not bar.setrandombarcolor then
						AllBars[count]:SetStatusBarColor(c.r, c.g, c.b, db.BarAlpha)
						bar.playertext:SetText(classes[CLASS_SORT_ORDER[randClass]])
					end
					bar.setrandombarcolor = true
				end			
				local range = bar.randomrange or math.random(db.range)
				bar.randomrange = range
				bar.range = range
				local val = range / db.range	
				if db.Reverse then val = 1 - val end
				bar:SetValue(val)				
				bar.distancetext:SetText(range)
				
				if db.ShowRaidIcon then
					if not bar.randomset then
						bar.raidicon:SetTexCoord(unpack(iconCoords[math.random(8)]))
					end
					bar.raidicon:Show()
					bar.randomset = true
				else
					bar.raidicon:Hide()
				end
				
				if db.ShowIcon then
					if not bar.randomIconset then
						bar.icon:SetTexCoord(unpack(ICON_COORDS[CLASS_SORT_ORDER[randClass]]))						
					end
					bar.randomIconset = true
					bar.icon:Show()
				else
					bar.icon:Hide()
				end
			end
		end
		
		table.sort(AllBars, function(a, b)
			if a.range and b.range then
				if a.range == b.range then return a.id < b.id end
				if db.CloseBottom then
					return a.range > b.range
				else
					return a.range < b.range
				end
			end
			if db.CloseBottom then
				if not a then return false end
				if not b then return true end
			else
				if not a then return true end
				if not b then return false end
			end
		end)
		
		for i, bar in pairs(AllBars) do			
			local t = i - 1
			bar.frame:ClearAllPoints()
			bar.frame:SetPoint("TOPLEFT", Window.title, "BOTTOMLEFT", 2, -(db.Spacing * t) - (db.NamesHeight * t))
			bar.frame:SetPoint("TOPRIGHT", Window.title, "BOTTOMRIGHT", -2, -(db.Spacing * t) - (db.NamesHeight * t))
			bar:ApplySettings()
		end
		
		for i = 1, #AllBars do
			if i <= db.NrNames and i <= count then
				AllBars[i]:Show()
			else
				AllBars[i].range = db.CloseBottom and -1 or 999999999999
				AllBars[i]:Hide()
				AllBars[i].randomset = nil
				AllBars[i].randomrange = nil	
				AllBars[i].setrandombarcolor = nil
				AllBars[i].randomIconset = nil
			end
		end
	end
end

function plugin:SetTitle(str)
	if str then
		Window:SetTitle(str)
	else
		Window:SetTitle(L.NAME .. " - " ..db.range.."y")
	end
end

function plugin:IncreaseRange()
	db.range = db.range + (IsShiftKeyDown() and 5 or 1)
	self:SetTitle()
end

function plugin:DecreaseRange()
	db.range = db.range - (IsShiftKeyDown() and 5 or 1)
	self:SetTitle()
end

function plugin:ResetFilters(clear)
	db.Filter.UseClassFilter = false
	db.Filter.UseIconFilter = false
	
	for class in pairs(LOCALIZED_CLASS_NAMES_MALE) do
		if clear then
			db.Filter.classes[class] = nil
		else
			db.Filter.classes[class] = true
		end
	end

	for i = 1, 8 do
		if clear then
			db.Filter.icons[i] = nil
		else
			db.Filter.icons[i] = true
		end
	end
	
	LocalFilter.UseNameFilter = false
	wipe(LocalFilter.FilteredName)
end

local KnownRangeChecks = {}
local FallBackChecks = {}
setmetatable(FallBackChecks, {
	__index = function(t, key)
		if key <= 10 then return 10
		elseif key >= 11 and key < 13 then return 11
		elseif key >= 13 and key <= 20 then return 15
		else return 28 end
	end
})
do
	local bandages = {21991, 34721, 38643, 34722, 34721, 34722}

	KnownRangeChecks[15] = function(uID)
		for i, v in ipairs(bandages) do
			if IsItemInRange(v, uID) == 1 then
				return true
			elseif IsItemInRange(v, uID) == 0 then
				return false
			end
		end
	end
	
	KnownRangeChecks[11] = function(uID)
		return CheckInteractDistance(uID, 2)
	end


	KnownRangeChecks[10] = function(uID)
		return CheckInteractDistance(uID, 3)
	end

	KnownRangeChecks[28] = function(uID)
		return CheckInteractDistance(uID, 4)
	end
end

function plugin:CheckRange(uID)	
	local d = Utils:GetDistanceAndAngleToUnit(uID)
	
	if not d then 		
		fallbackRange = FallBackChecks[db.range]		
		return KnownRangeChecks[fallbackRange](uID), fallbackRange
	end
	
	return d < db.range, d
end

function plugin:ApplySettings()
	frame:ClearAllPoints()
	if db.GrowUp then
		frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
	else
		frame:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
	end
end

--- Show range check
-- @param show Indicates if the window should be shown or hidde (true/false)
function plugin:RW_ShowRangeCheck(message, show, range, force)
	if not Window then self:CreateWindow() end
	
	if range and (not db.Locked or force) then
		db.range = range
		self:SetTitle()
	end
	
	if show then
		Window:Show()
	elseif not (db.Locked or db.NoAutoHide) or force then
		Window:Hide()
	end
end

function plugin:RW_HideRangeCheck(message, forced)
	if not Window then return end
	if not (db.Locked or db.NoAutoHide) or force then
		Window:Hide()
	end
end

function plugin:RW_RangeFilterClear(message)
	if not db.Locked then
		self:ResetFilters(true)
	end
end

function plugin:RW_RangeFilterAddIcon(message, clearOld, ...)
	if db.Locked then return end
	for i = 1, select("#", ...) do
		local val = select(i, ...)
		local icon
		if type(val) == "string" then
			icon = Utils.Constants.RaidIconNameToNumber[val]
		elseif type(val) == "number" then
			icon = val
		end
		
		if icon then
			db.Filter.icons[icon] = true
		end
	end
end

function plugin:RW_RangeFilterAddName(message, clearOld, ...)
	if db.Locked then return end
	for i = 1, select("#", ...) do
		local name = select(i, ...)
		LocalFilter.FilteredName[name] = true
	end
end

function plugin:RW_RangeFilterRemoveName(message, ...)
	if db.Locked then return end
	for i = 1, select("#", ...) do
		local name = select(i, ...)
		LocalFilter.FilteredName[name] = nil
	end
end

function plugin:RW_RangeFilterIcons(message, turnOn)
	if not db.Locked then
		db.Filter.UseIconFilter = turnOn
		
		self:Reset()
	end
end

function plugin:RW_RangeFilterClasses(message, turnOn)
	if not db.Locked then
		db.Filter.UseClassFilter = turnOn
		
		self:Reset()
	end
end

function plugin:RW_RangeFilterNames(message, turnOn)
	if not db.Locked then
		LocalFilter.UseNameFilter = turnOn
		
		self:Reset()
	end
end

function plugin:RW_ConfigMode(message, start)
	if not Window then self:CreateWindow() end
	if start then
		InConfigMode = true
		Anchor:Show()
		Window:Show()
	else
		InConfigMode = nil
		Anchor:Hide()
		Window:Hide()
		for i = 1, #AllBars do
			AllBars[i]:Hide()
			AllBars[i].randomset = false
			AllBars[i].randomrange = nil		
		end
	end
end

dragStart = function(self)
	self:StartMoving()
end
dragStop = function(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	x = x * scale
	y = y * scale
	
	db.x = x
	db.y = y
	
	plugin:SendMessage("RW_GUIValuesChanged")
	if ConfigWindow then
		plugin:OpenConfigWindow(true)
	end
end

------------------------------------
-- Options
do
	function plugin:GetOptions()
		return LB.PLUGINS, LB.ALERT_PLUGINS, L.NAME, self.GetGUI, [[Interface\Icons\INV_Misc_Map_01]]
	end

	local function changeCallback()
		plugin:Reset()
	end
	
	-- Locals
	local LastSelectedTab
	local SideList = {
		["LEFT"] = "Left",
		["RIGHT"] = "Right",
	}
	-- Functions
	local TabSelected

	function plugin:GetGUI()
		local self = self or plugin
		local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
		local AceGUI = LibStub("AceGUI-3.0")
		
		local group, label, slider, check, igroup, button, color, select
		
		group = AceGUI:Create("ScrollFrame")
		group:SetLayout("Flow")
		
		label = UIF:MainTitleLabel(L.NAME)
		group:AddChild(label)
		
		igroup = UIF:InlineGroup("")
		group:AddChild(igroup)	
		
		label = UIF:NormalText(L.DESC)
		label:SetImage([[Interface\Icons\INV_Misc_Map_01]])
		igroup:AddChild(label)
		
		group:AddChild(UIF:CheckBox(L.LOCK, db, "Locked", function()
			if LockButton then
				if db.Locked then
					LockButton.texture:SetVertexColor(1, 0, 0)
				else
					LockButton.texture:SetVertexColor(unpack(RW.db.profile.WindowButtonColor))
				end
			end
		end, L.LOCK_DESC, 0.5))	
		group:AddChild(UIF:CheckBox(L.NO_AUTO_HIDE, db, "NoAutoHide", nil, L.NO_AUTO_HIDE_DESC, 0.5))
		group:AddChild(UIF:Button(LB.SHOW, function() plugin:SendMessage("RW_ShowRangeCheck", true) end))		
		group:AddChild(UIF:Button(LB.HIDE, function() plugin:SendMessage("RW_ShowRangeCheck", false) end))
		
		button = UIF:Button(LB.RESET, function() 
			plugin.db:ResetProfile() 
			if ConfigWindow then
				plugin:OpenConfigWindow(true)
			end
		end)
		button:SetUserData("confirm", true)
		button:SetUserData("TriggerGlobalUpdate", true)
		group:AddChild(button)
		
		local tgroup = UIF:TabGroup1()
		group:AddChild(tgroup)
		tgroup:SetTabs(
		{
			{text = L.FILTER, value = "Filter"},
			{text = LB.SIZE.." & " ..LB.POSITION, value = "Size"},
			{text = LB.APPEARANCE, value = "Appearance"},
		})
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastSelectedTab or "Filter")
		
		group:FixScroll()
		
		return group
	end
	
	TabSelected = function(self, event, tab)
		local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
		local AceGUI = LibStub("AceGUI-3.0")
		
		LastSelectedTab = tab
		
		self:ReleaseChildren()
		
		if tab == "Filter" then
			local button = UIF:Button(L.RESET_FILTERS, function()
				plugin:ResetFilters()
				if ConfigWindow then
					plugin:OpenConfigWindow(true)
				end
			end)
			button:SetUserData("TriggerGlobalUpdate", true)
			self:AddChild(button)
		
			local group = UIF:InlineGroup2(LB.CLASS)
			self:AddChild(group)			
			
			local check = UIF:CheckBox(L.USE_CLASS_FILTER, db.Filter, "UseClassFilter", changeCallback)
			group:AddChild(check)
			
			for class, localClass in pairs(LOCALIZED_CLASS_NAMES_MALE) do
				local check = UIF:CheckBox(localClass, db.Filter.classes, class)
				check:SetImage("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes", unpack(ICON_COORDS[class]))
				check:SetRelativeWidth(0.33)
				group:AddChild(check)
			end
			
			group = UIF:InlineGroup2(LB.RAID_ICON)
			self:AddChild(group)
			
			check = UIF:CheckBox(L.USE_RAIDICON_FILTER, db.Filter, "UseIconFilter", changeCallback)
			group:AddChild(check)
			
			for i = 1, 8 do 
				check = UIF:CheckBox("", db.Filter.icons, i)
				check:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[i]))
				check:SetRelativeWidth(0.33)
				group:AddChild(check)
			end
			
			group = UIF:InlineGroup2(LB.PARTY)
			self:AddChild(group)
			
			group:AddChild(UIF:Text1(L.NAME_FILET_DESC))
			
			check = UIF:CheckBox(L.USE_NAME_FILTER, LocalFilter, "UseNameFilter", changeCallback)
			group:AddChild(check)
			
			for name, data in Party:PartyIterator() do 
				if name ~= UnitName("player") then
					check = UIF:CheckBox(name, LocalFilter.FilteredName, name)
					check:SetRelativeWidth(0.33)
					group:AddChild(check)
				end
			end
		
		elseif tab == "Size" then
			local group = UIF:InlineGroup2(LB.SIZE)
			self:AddChild(group)
			
			local group2 = UIF:InlineGroup3(L.WINDOW)
			group:AddChild(group2)
			
			group2:AddChild(UIF:Slider(LB.WIDTH, db, "Width", 120, 500, 1, changeCallback, 0.5))
			
			group2 = UIF:InlineGroup3(L.BARS)
			group:AddChild(group2)
			
			group2:AddChild(UIF:Slider(LB.HEIGHT, db, "NamesHeight", 5, 60, 1, changeCallback, 0.33))
			group2:AddChild(UIF:Slider(L.NR_NAMES, db, "NrNames", 1, 40, 1, changeCallback, 0.33))
			group2:AddChild(UIF:Slider(LB.SPACING, db, "Spacing", 0, 20, 1, changeCallback, 0.33))
			
			group = UIF:InlineGroup2(LB.POSITION)
			self:AddChild(group)
			
			group:AddChild(UIF:Slider("X", db, "x", -floor(GetScreenWidth() * UIParent:GetEffectiveScale()), floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 1, changeCallback, 0.5))
			group:AddChild(UIF:Slider("Y", db, "y", -floor(GetScreenHeight() * UIParent:GetEffectiveScale()), floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 1, changeCallback, 0.5))
		
		elseif tab == "Appearance" then
			-- Window
			local group = UIF:InlineGroup2(L.WINDOW)
			self:AddChild(group)
			
			group:AddChild(UIF:ColorSelect(LB.BORDER_COLOR, db, "BorderColor", changeCallback, true, 0.33))			
			group:AddChild(UIF:LSMDropdown("border", LB.BORDER_TEXTURE, db, "Border", changeCallback, 0.33))			
			group:AddChild(UIF:Slider(LB.BORDER_SIZE, db, "BorderWidth", 1, 40, 1, changeCallback, 0.33))			
			group:AddChild(UIF:Slider(LB.INSET, db, "Inset", 0, 50, 1, changeCallback, 0.5))
			group:AddChild(UIF:Slider(LB.ALPHA, db,"Alpha", 0, 1, 0.01, changeCallback, 0.5))
			
			-- Icons
			group = UIF:InlineGroup2(L.ICONS)
			self:AddChild(group)
			
			group:AddChild(UIF:CheckBox(L.SHOW_RAID_ICON, db, "ShowRaidIcon", changeCallback, nil, 0.33))			
			group:AddChild(UIF:CheckBox(L.SHOW_CLASS_ICON, db, "ShowIcon", changeCallback, nil, 0.33))			
			group:AddChild(UIF:Dropdown(L.ICON_SIDE, db, "IconSide", changeCallback, SideList, 0.33))
			
			-- Bars
			group = UIF:InlineGroup2(L.BARS)
			self:AddChild(group)
			
			check = UIF:CheckBox(L.USE_CLASS_COLORS, db, "UseClassColor", function(self)
				changeCallback()
				self:GetUserData("color"):SetDisabled(db.UseClassColor)
			end)			
			check:SetRelativeWidth(0.33)
			group:AddChild(check)
			
			color = UIF:ColorSelect(LB.BAR_COLOR, db, "BarColor", changeCallback, true)
			color:SetDisabled(db.UseClassColor)
			color:SetRelativeWidth(0.33)
			check:SetUserData("color", color)
			group:AddChild(color)	
			
			drop = UIF:LSMDropdown("statusbar", LB.BAR_TEXTURE, db, "BarTexture", changeCallback)
			drop:SetRelativeWidth(0.33)
			group:AddChild(drop)	

			group:AddChild(UIF:CheckBox(L.REVERSE_BARS, db, "Reverse", nil, L.REVERSE_BARS_DESC, 0.5))
			group:AddChild(UIF:CheckBox(L.SHOW_CLOSE, db, "CloseBottom", nil, L.SHOW_CLOSE_DESC, 0.5))

			-- Fonts
			group = UIF:InlineGroup2("Fonts")
			self:AddChild(group)
			
			-- Names
			local group2 = UIF:InlineGroup3(L.NAMES)
			group:AddChild(group2)
			
			drop = UIF:LSMDropdown("font", LB.FONT, db, "NameFont", changeCallback)
			group2:AddChild(drop)
			
			drop = UIF:Dropdown(LB.FONT_OUTLINE, db, "NameFontFlags", changeCallback, Utils.Constants.FontFlagList)
			group2:AddChild(drop)
			
			slider = UIF:Slider(LB.FONT_SIZE, db, "NameFontSize", 1, 50, 1, changeCallback)
			group2:AddChild(slider)
			
			color = UIF:ColorSelect(LB.COLOR, db, "NameFontColor", changeCallback)
			group2:AddChild(color)
			
			-- Distance
			group2 = UIF:InlineGroup3(L.DISTANCE)
			group:AddChild(group2)
			
			drop = UIF:LSMDropdown("font", LB.FONT, db, "DistanceFont", changeCallback)
			group2:AddChild(drop)
			
			drop = UIF:Dropdown(LB.FONT_OUTLINE, db, "DistanceFontFlags", changeCallback, Utils.Constants.FontFlagList)
			group2:AddChild(drop)
			
			slider = UIF:Slider(LB.FONT_SIZE, db, "DistanceFontSize", 1, 50, 1, changeCallback)
			group2:AddChild(slider)
			
			color = UIF:ColorSelect(LB.COLOR, db, "DistanceFontColor", changeCallback)
			group2:AddChild(color)	
		end
		
		self.parent:DoLayout()
		self:DoLayout()
	end
end
